// ==============================================================
// Fuel MFD - fuel management MFD for Orbiter
//
// FuelMFD.cpp
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "orbitersdk.h"
#include "OrbiterSoundSDK.h"
#include "MFDlib.h"
#include "FuelMFD.h"

// ==============================================================
// Global variables

static int g_FuelMFD;			// registered MFD mode in Orbiter

static FUEL_TRANSFER Transfer;

static struct VESSELSTATUS2::FUELSPEC fs[MAX_FUEL_TANKS]; // propellant list

static char *sPumpMode[] = { "Cross-Feed", "Load", "Unload", "Vent" };
static char *sDescMode[] = { " (SRC => TGT)", " (SRC => TGT)", " (TGT => SRC)", " (TGT => VENT)" };
static char *sPumpRate[] = { "Off", "Low", "Medium", "High" };

// ==============================================================
// global routines

bool PumpingAllowed(const FUEL_TRANSFER &ft)
{
	bool CanPump;
	VESSELSTATUS2 vs2;
	
	// assume everything okay
	CanPump = true;
	
	// target vessel must have a target tank
	vs2.version = 2;
	vs2.flag = VS_FUELLIST;
	vs2.fuel = &fs[0];
	vs2.thruster = NULL;
	vs2.dockinfo = NULL;
	ft.TargetTank.vessel->GetStatusEx(&vs2);
	if (vs2.nfuel == 0)
		CanPump = false;
	else 
		// must have a source vessel (unless venting)
		if (!ft.SourceTank.vessel)
			CanPump = (ft.PumpMode == PM_VENT);
		else {
			//delete []vs2.fuel; // clear old array
			// source vessel must have a source tank
			vs2.version = 2;
			vs2.flag = VS_FUELLIST;
			vs2.fuel = &fs[0];
			vs2.thruster = NULL;
			vs2.dockinfo = NULL;
			ft.SourceTank.vessel->GetStatusEx(&vs2);
			if (ft.SourceTank.propnum >= (int)vs2.nfuel)
				CanPump = false;
			else // can't be EXACTLY THE SAME tank
				if ((ft.SourceTank.vessel == ft.TargetTank.vessel) && (ft.SourceTank.propnum == ft.TargetTank.propnum))
					CanPump = false;
		}
	//delete []vs2.fuel;
	return CanPump;
}

// ==============================================================
// API interface

DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	static char *name = "Fuel Management";
	MFDMODESPEC spec;

	//MessageBox(NULL, "DLLInit", "FuelMFD", MB_OK);
	spec.name    = name;
	spec.key     = OAPI_KEY_F;	// activation hotkey
	spec.msgproc = FuelMFD::MsgProc;

	g_FuelMFD = oapiRegisterMFDMode (spec);

	Transfer.PumpMode = PM_CROSSFEED;
	Transfer.PumpRate = PR_OFF;
	Transfer.DockPort = 0;
	Transfer.TargetTank.vessel = NULL;
	Transfer.TargetTank.propnum = 0;
	Transfer.SourceTank = Transfer.TargetTank;
}

DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
	//MessageBox(NULL, "DLLExit", "FuelMFD", MB_OK);
	oapiUnregisterMFDMode (g_FuelMFD);
}

DLLCLBK void opcOpenRenderViewport (void)
{
	//MessageBox(NULL, "OpenRenderViewport", "FuelMFD", MB_OK);
}

DLLCLBK void opcCloseRenderViewport (void)
{
	//MessageBox(NULL, "CloseRenderViewport", "FuelMFD", MB_OK);
}

DLLCLBK void opcFocusChanged (OBJHANDLE new_focus, OBJHANDLE old_focus)
{
	//MessageBox(NULL, "FocusChanged", "FuelMFD", MB_OK);
}

DLLCLBK void opcTimeAccChanged (double new_warp, double old_warp)
{
	//MessageBox(NULL, "TimeAccChanged", "FuelMFD", MB_OK);
}

DLLCLBK void opcTimestep (double SimT, double SimDT, double mjd)
{
	static double FlowRate[] = { 0.0, 1.0, 10.0, 100.0 };
	FUEL_TANK FromTank, ToTank;
	PROPELLANT_HANDLE ph1, ph2;
	double pAmount, pMass1, pMass2, pMaxMass;

	if ((Transfer.PumpRate != PR_OFF) && PumpingAllowed(Transfer)) {
		// are we loading or unloading our ship
		if ((Transfer.PumpMode == PM_UNLOAD) || (Transfer.PumpMode == PM_VENT)) { 
			// TGT => SRC
			FromTank = Transfer.TargetTank;
			ToTank = Transfer.SourceTank;
		} else { 
			// SRC => TGT
			FromTank = Transfer.SourceTank;
			ToTank = Transfer.TargetTank;
		}

		// how much fuel do we want this timestep?
		pAmount = SimDT * FlowRate[Transfer.PumpRate];

		// will we empty the FROM tank this timestep?
		ph1 = oapiGetPropellantHandle(FromTank.vessel->GetHandle(), FromTank.propnum);
		if (ph1)
			pMass1 = FromTank.vessel->GetPropellantMass(ph1);
		else
			pMass1 = 0;
		if (pAmount >= pMass1) {
			pAmount = pMass1; // nearly empty...take all of it
			Transfer.PumpRate = PR_OFF; // auto-shutdown when empty
		}

		// will we fill the TO tank in this timestep?
		if (Transfer.PumpMode != PM_VENT) {
			ph2 = oapiGetPropellantHandle(ToTank.vessel->GetHandle(), ToTank.propnum);
			if (ph2) {
				pMass2 = ToTank.vessel->GetPropellantMass(ph2);
				pMaxMass = Transfer.TargetTank.vessel->GetPropellantMaxMass(ph2);
			} else {
				pMass2 = 0;
				pMaxMass = 0;
			}
			if (pMass2+pAmount >= pMaxMass) {
				pAmount = pMaxMass - pMass2; // nearly full...just top it up
				Transfer.PumpRate = PR_OFF; // auto-shutdown when full
			}
		}

		// take the fuel OUT of the FROM tank
		if (ph1)
			FromTank.vessel->SetPropellantMass(ph1, pMass1 - pAmount);

		// put the fuel INTO the TO tank
		if ((Transfer.PumpMode != PM_VENT) && (ph2))
			ToTank.vessel->SetPropellantMass(ph2, pMass2 + pAmount);
	}
}


// ==============================================================
// Fuel MFD implementation

FuelMFD::FuelMFD (DWORD w, DWORD h, VESSEL *vessel)
: MFD (w, h, vessel)
{
	//MessageBox(NULL, "Constructor", "FuelMFD", MB_OK);
	HelpPage = 0;
	if (Transfer.TargetTank.vessel != vessel) {
		Transfer.PumpMode = PM_CROSSFEED;
		Transfer.PumpRate = PR_OFF;
		Transfer.DockPort = 0;
		Transfer.TargetTank.vessel = vessel;
		Transfer.TargetTank.propnum = 0;
		Transfer.SourceTank = Transfer.TargetTank;
	}
	ConnectToOrbiterSoundDLL();
	RequestLoadVesselWave(1,"Sound\\FuelMFD\\PumpOn.wav");
	RequestLoadVesselWave(2,"Sound\\FuelMFD\\PumpOff.wav");
}

FuelMFD::~FuelMFD ()
{
	//MessageBox(NULL, "Destructor", "FuelMFD", MB_OK);
}

// message parser
int FuelMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int)(new FuelMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	}
	return 0;
}

// simulate keypress for panel interface
bool FuelMFD::ConsumeButton (int bt, int event)
{
	static const DWORD btkey[] = {
		OAPI_KEY_M,
		OAPI_KEY_EQUALS,
		OAPI_KEY_MINUS,
		OAPI_KEY_T,
		OAPI_KEY_S,
		OAPI_KEY_D,
		OAPI_KEY_SLASH
	};
	if (!(event & PANEL_MOUSE_DOWN))
		return false;
	if (bt < sizeof(btkey)/sizeof(btkey[0]))
		return ConsumeKeyBuffered (btkey[bt]);
	else
		return false;
}

char *FuelMFD::ButtonLabel (int bt)
{
	char *label[] = {
		"MOD", "PR+", "PR-", "TGT", "SRC", "DCK", "HLP"
	};
	return (bt < sizeof(label)/sizeof(label[0]) ? label[bt] : 0);
}

int FuelMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	static const MFDBUTTONMENU mnu[] = {
		{"Pump Mode", NULL, 'M'},
		{"Increase Rate", NULL, '+'},
		{"Decrease Rate", NULL, '-'},
		{"Target Tank", NULL, 'T'},
		{"Source Tank", NULL, 'S'},
		{"Docking Port", NULL, 'D'},
		{"Help", NULL, '?'}
	};
	if (menu) *menu = mnu;
	return sizeof(mnu)/sizeof(mnu[0]);
}

void FuelMFD::SetTransferSource(void)
{
	DOCKHANDLE dHandle;
	OBJHANDLE oHandle;
	VESSEL *oldVessel;
	
	if ((Transfer.PumpMode == PM_CROSSFEED) || (Transfer.PumpMode == PM_VENT)) return;
	
	// does target vessel have any docking ports?
	if (Transfer.TargetTank.vessel->DockCount() == 0) {
		Transfer.SourceTank.vessel = NULL;
		Transfer.SourceTank.propnum = 0;
		Transfer.PumpRate = PR_OFF;
		return;
	}

	// get docking port details
	dHandle = Transfer.TargetTank.vessel->GetDockHandle(Transfer.DockPort);
	
	// it there anything docked on selected port?
	oHandle = Transfer.TargetTank.vessel->GetDockStatus(dHandle);
	if (oHandle) {
		oldVessel = Transfer.SourceTank.vessel;
		Transfer.SourceTank.vessel = oapiGetVesselInterface(oHandle);
		
		// has docking status changed? reset tank & turn pumps off
		if (oldVessel != Transfer.SourceTank.vessel) {
			Transfer.SourceTank.propnum = 0;
			Transfer.PumpRate = PR_OFF;
		}
	} else {
		Transfer.SourceTank.vessel = NULL;
		Transfer.SourceTank.propnum = 0;
		Transfer.PumpRate = PR_OFF;
	}
}

bool FuelMFD::ConsumeKeyBuffered(DWORD key)
{
	VESSELSTATUS2 vs2;

	switch(key) {
	case OAPI_KEY_M:
		if (HelpPage) return true;
		Transfer.PumpMode = PUMPMODE((int(Transfer.PumpMode) + 1) % 4);
		if (Transfer.PumpMode == PM_CROSSFEED)
			Transfer.SourceTank = Transfer.TargetTank;
		if (Transfer.PumpMode == PM_VENT)
			Transfer.SourceTank.vessel = NULL;
		Transfer.PumpRate = PR_OFF;	// shut down pumps on mode change
		InvalidateDisplay();		// force immediate screen refresh
		return true;
	case OAPI_KEY_EQUALS:
		if (HelpPage) return true;
		// validate source vs. target before starting pumps
		if (PumpingAllowed(Transfer)) {
			Transfer.PumpRate = PUMPRATE(min(int(Transfer.PumpRate)+1, int(PR_HIGH)));
			PlayVesselWave(1);
		} else
			Transfer.PumpRate = PR_OFF;
		InvalidateDisplay();	// force immediate screen refresh
		return true;
	case OAPI_KEY_MINUS:
		if (HelpPage) return true;
		if (PumpingAllowed(Transfer)) {
			Transfer.PumpRate = PUMPRATE(max(int(Transfer.PumpRate)-1, int(PR_OFF)));
			PlayVesselWave(2);
		} else
			Transfer.PumpRate = PR_OFF;
		InvalidateDisplay();	// force immediate screen refresh
		return true;
	case OAPI_KEY_T:
		if (HelpPage) return true;
		vs2.version = 2;
		vs2.flag = VS_FUELLIST;
		vs2.fuel = &fs[0];
		vs2.thruster = NULL;
		vs2.dockinfo = NULL;
		Transfer.TargetTank.vessel->GetStatusEx(&vs2);
		if (vs2.nfuel > 1) {
			Transfer.TargetTank.propnum = (Transfer.TargetTank.propnum+1) % vs2.nfuel;
			Transfer.PumpRate = PR_OFF;	// shut down pumps on target tank change
			InvalidateDisplay();		// force immediate screen refresh
		}
		//delete []vs2.fuel;
		return true;
	case OAPI_KEY_S:
		if (HelpPage) return true;
		if (Transfer.SourceTank.vessel) {
			vs2.version = 2;
			vs2.flag = VS_FUELLIST;
			vs2.fuel = &fs[0];
			vs2.thruster = NULL;
			vs2.dockinfo = NULL;
			Transfer.SourceTank.vessel->GetStatusEx(&vs2);
			if (vs2.nfuel > 1) {
				Transfer.SourceTank.propnum = (Transfer.SourceTank.propnum+1) % vs2.nfuel;
				Transfer.PumpRate = PR_OFF;	// shut down pumps on source tank change
				InvalidateDisplay();		// force immediate screen refresh
			}
			//delete []vs2.fuel;
		}
		return true;
	case OAPI_KEY_D:
		if (HelpPage) return true;
		if (pV->DockCount() > 1) {
			Transfer.DockPort = (Transfer.DockPort+1) % pV->DockCount();
			Transfer.PumpRate = PR_OFF;	// shut down pumps on dock port change
			InvalidateDisplay();		// force immediate screen refresh
		}
		return true;
	case OAPI_KEY_SLASH:
		HelpPage = (HelpPage+1) % 2;
		InvalidateDisplay();
		return true;
	default:
		return false;
	}
}

void FuelMFD::ShowFuelInfo(const HDC hDC)
{
	
	// display general info
	TextXY(hDC, WHITE, " %s (%s)", pV->GetName(), pV->GetClassName());
	
	TextXY(hDC, 0, 3, GREEN, "Pump Mode: ");
	TextXY(hDC, BRIGHTGREEN, sPumpMode[Transfer.PumpMode]);
	TextXY(hDC, GREEN, sDescMode[Transfer.PumpMode]);
	
	TextXY(hDC, 0, 6, GREEN, "Pump Rate: ");
	TextXY(hDC, BRIGHTGREEN, sPumpRate[Transfer.PumpRate]);
}

void FuelMFD::ShowFuelGauge(const HDC hDC, const int x, const int y, const FUEL_TANK &ft)
{
	PROPELLANT_HANDLE ph;
	double pMass, pMaxMass, pPercentage;
	int BarLength;
	COLORREF BarColor;
	POINT p;
	HBRUSH hOldBrush, hNewBrush;

	ph = oapiGetPropellantHandle(ft.vessel->GetHandle(), ft.propnum);
	pMass = ft.vessel->GetPropellantMass(ph);
	pMaxMass = ft.vessel->GetPropellantMaxMass(ph);
	SetTextXY(hDC, x, y);	// move cursor
	p = GetTextPos(hDC);	// get pixel position
	Rectangle(hDC, p.x, p.y+ch/2, W-p.x, p.y+ch*2);

	// set bar color
	BarColor = LIGHTGREEN;
	if (pMaxMass < 0.01) { // dummy tank?
		BarLength = 0;
		pPercentage = 0.0;
	} else {
		BarLength = int(pMass / pMaxMass * (W-2*p.x));
		pPercentage = pMass / pMaxMass * 100.0;
		if (pPercentage < 50) BarColor = LIGHTYELLOW;
		if (pPercentage < 10) BarColor = LIGHTRED;
	}

	hNewBrush = CreateSolidBrush(BarColor);
	hOldBrush = (HBRUSH) SelectObject(hDC, hNewBrush);
	Rectangle(hDC, p.x, p.y+ch/2, p.x+BarLength, p.y+ch*2);
	SelectObject(hDC, hOldBrush);
	DeleteObject(hNewBrush);

	SetTextPos(hDC, p.x, p.y+ch/2);
	TextXY(hDC, 1, USE_CURRENT, BRIGHTWHITE, pMass < 1000 ? "%.1f kg" : "%.0f kg", pMass);
	TextXY(hDC, 15, USE_CURRENT, "%.2f%%", pPercentage);
	TextXY(hDC, 27, USE_CURRENT, pMaxMass < 1000 ? "%6.1f kg" : "%6.0f kg", pMaxMass);
}

void FuelMFD::ShowTargetTank(const HDC hDC)
{
	VESSELSTATUS2 vs2;
	
	vs2.version = 2;
	vs2.flag = VS_FUELLIST;
	vs2.fuel = &fs[0];
	vs2.thruster = NULL;
	vs2.dockinfo = NULL;
	Transfer.TargetTank.vessel->GetStatusEx(&vs2);
	
	if (vs2.nfuel == 0) {
		TextXY(hDC, 0, 13, BRIGHTWHITE, RED, " NO TARGET TANK ");
	} else {
		// sanity check for "removeable" tanks
		if (Transfer.TargetTank.propnum >= (int)vs2.nfuel) {
			Transfer.TargetTank.propnum = 0; // switch to first tank
			Transfer.PumpRate = PR_OFF; // shut down any transfers
		}
		TextXY(hDC, 0, 13, GREEN, "Target Tank ");
		TextXY(hDC, BRIGHTGREEN, "%d", Transfer.TargetTank.propnum+1);
		TextXY(hDC, GREEN, " (of %d)", vs2.nfuel);
		ShowFuelGauge(hDC, 0, 14, Transfer.TargetTank);
	}
	//delete []vs2.fuel;
}

void FuelMFD::ShowDockingPort(const HDC hDC)
{
	// can't load/unload without a docking port
	if (pV->DockCount() == 0) {
		TextXY(hDC, 0, 20, BRIGHTWHITE, RED, " NO DOCKING PORTS ");
		return;
	}
	
	// display current docking port & vessel
	TextXY(hDC, 0, 20, GREEN, "Docking Port ");
	TextXY(hDC, BRIGHTGREEN, "%d", Transfer.DockPort+1);
	TextXY(hDC, GREEN, " (of %d)", pV->DockCount());
	
	// anything docked there?
	if (Transfer.SourceTank.vessel) {
		TextXY(hDC, 0, 21, "Docked To: ");
		TextXY(hDC, BRIGHTGREEN, Transfer.SourceTank.vessel->GetName());
	} else
		TextXY(hDC, 0, 21, BRIGHTWHITE, YELLOW, " NOT DOCKED ");
}

void FuelMFD::ShowSourceTank(const HDC hDC)
{
	VESSELSTATUS2 vs2;

	vs2.version = 2;
	vs2.flag = VS_FUELLIST;
	vs2.fuel = &fs[0];
	vs2.thruster = NULL;
	vs2.dockinfo = NULL;
	if (Transfer.PumpMode == PM_CROSSFEED) {
		// display source tank in same vessel
		Transfer.SourceTank.vessel->GetStatusEx(&vs2);
		// sanity check for "removeable" tanks
		if (Transfer.SourceTank.propnum >= (int)vs2.nfuel) {
			Transfer.SourceTank.propnum = 0; // switch to first tank
			Transfer.PumpRate = PR_OFF; // shut down any transfers
		}
		
		if (vs2.nfuel == 0) {
			TextXY(hDC, 0, 17, BRIGHTWHITE, RED, " NO SOURCE TANK ");
		} else if (vs2.nfuel == 1) {
			TextXY(hDC, 0, 17, BRIGHTWHITE, YELLOW, " SINGLE TANK ONLY ");
		} else{
			TextXY(hDC, 0, 17, GREEN, "Source Tank ");
			TextXY(hDC, BRIGHTGREEN, "%d", Transfer.SourceTank.propnum+1);
			TextXY(hDC, GREEN, " (of %d)", vs2.nfuel);
			ShowFuelGauge(hDC, 0, 18, Transfer.SourceTank);
		}
	} else if (Transfer.PumpMode == PM_VENT)
		TextXY(hDC, 0, 17, BRIGHTWHITE, YELLOW, " OVERBOARD VENT ");
	else {
		// display source tank in different vessel
		ShowDockingPort(hDC);
		// if not docked, we're done
		if (!Transfer.SourceTank.vessel) return;

		Transfer.SourceTank.vessel->GetStatusEx(&vs2);
		// sanity check for "removeable" tanks
		if (Transfer.SourceTank.propnum >= (int)vs2.nfuel) {
			Transfer.SourceTank.propnum = 0; // switch to first tank
			Transfer.PumpRate = PR_OFF; // shut down any transfers
		}
		if (vs2.nfuel == 0) {
			TextXY(hDC, 0, 17, BRIGHTWHITE, RED, " NO SOURCE TANK ");
		} else{
			TextXY(hDC, 0, 17, GREEN, "Source Tank ");
			TextXY(hDC, BRIGHTGREEN, "%d", Transfer.SourceTank.propnum+1);
			TextXY(hDC, GREEN, " (of %d)", vs2.nfuel);
			ShowFuelGauge(hDC, 0, 18, Transfer.SourceTank);
		}
	}
	//delete []vs2.fuel;
}

void FuelMFD::ShowHelp(const HDC hDC)
{
	switch (HelpPage) {
	case 1:
		TextXY(hDC, " Version %s", MFDversion);
		TextXY(hDC, 0, 2, BRIGHTGREEN, "CROSS-FEED");
		TextXY(hDC, GREEN, " allows you to transfer");
		TextXY(hDC, 0, 3, "fuel between internal tanks (top up ");
		TextXY(hDC, 0, 4, "those RCS thrusters!)");
		TextXY(hDC, 0, 6, BRIGHTGREEN, "LOAD");
		TextXY(hDC, GREEN, " transfers fuel from a vessel");
		TextXY(hDC, 0, 7, "docked to yours into one of your");
		TextXY(hDC, 0, 8, "internal tanks");
		TextXY(hDC, 0, 10, BRIGHTGREEN, "UNLOAD");
		TextXY(hDC, GREEN, " transfers fuel from one of");
		TextXY(hDC, 0, 11, "your internal tanks into a vessel");
		TextXY(hDC, 0, 12, "docked to yours");
		TextXY(hDC, 0, 14, BRIGHTGREEN, "VENT");
		TextXY(hDC, GREEN, " dumps your fuel overboard in");
		TextXY(hDC, 0, 15, "emergency situations");
		break;
	}
}

void FuelMFD::WriteStatus (FILEHANDLE scn) const
{
	//MessageBox(NULL, "WriteStatus", "FuelMFD", MB_OK);
	char buf[80];

	oapiWriteScenario_int(scn, "PUMPMODE", int(Transfer.PumpMode));
	oapiWriteScenario_int(scn, "PUMPRATE", int(Transfer.PumpRate));
	oapiWriteScenario_int(scn, "DOCKPORT", Transfer.DockPort);
	oapiWriteScenario_int(scn, "TARGETTANK", Transfer.TargetTank.propnum);
	if (Transfer.SourceTank.vessel) {
		wsprintf(buf, "%d:%s", Transfer.SourceTank.propnum, Transfer.SourceTank.vessel->GetName());
		oapiWriteScenario_string(scn, "SOURCETANK", buf);
	} 

	// reset fuel system for next scenario
	Transfer.PumpMode = PM_CROSSFEED;
	Transfer.PumpRate = PR_OFF;
	Transfer.DockPort = 0;
	Transfer.TargetTank.vessel = NULL;
	Transfer.TargetTank.propnum = 0;
	Transfer.SourceTank = Transfer.TargetTank;
}

void FuelMFD::ReadStatus (FILEHANDLE scn)
{
	//MessageBox(NULL, "ReadStatus", "FuelMFD", MB_OK);
	char buf[80];
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "PUMPMODE", 8))
			sscanf (line+8, "%d", &Transfer.PumpMode);
		else if (!strnicmp (line, "PUMPRATE", 8))
			sscanf (line+8, "%d", &Transfer.PumpRate);
		else if (!strnicmp (line, "DOCKPORT", 8))
			sscanf (line+8, "%d", &Transfer.DockPort);
		else if (!strnicmp (line, "TARGETTANK", 10))
			sscanf (line+10, "%d", &Transfer.TargetTank.propnum); // vessel set in constructor
		else if (!strnicmp (line, "SOURCETANK", 10)) {
			sscanf (line+10, "%d:%s", &Transfer.SourceTank.propnum, buf);
			Transfer.SourceTank.vessel = oapiGetVesselInterface(oapiGetVesselByName(buf));
		}
	}
}

void FuelMFD::Update (HDC hDC)
{
	MFDUpdate(hDC, cw, ch);
	Title(hDC, "Fuel Mgt");

	if (HelpPage)
		ShowHelp(hDC);
	else {
		// refresh source vessel details (in case we've undocked)
		SetTransferSource();
		ShowFuelInfo(hDC);
		ShowTargetTank(hDC);
		ShowSourceTank(hDC);
	}
}

void FuelMFD::StoreStatus(void) const
{
	//MessageBox(NULL, "StoreStatus", "FuelMFD", MB_OK);
}

void FuelMFD::RecallStatus(void)
{
	//MessageBox(NULL, "RecallStatus", "FuelMFD", MB_OK);
}
